==> klayout-0.17.1.tar.gz: klayout-0.17.1/src/gsiDeclEdt.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "gsi.h"
#include "edtService.h"
#include "layLayoutView.h"

namespace edt
{

static const db::Instance &inst (const edt::ObjectInstPath *p) 
{
  if (! p->is_cell_inst ()) {
    throw tl::Exception ("Selection does not represent an instance in 'inst' method");
  }
  return p->back ().inst_ptr;
}

static unsigned int path_length (const edt::ObjectInstPath *p)
{
  //  HINT: this method is not quite fast - the path is a list
  return std::distance (p->begin (), p->end ());
}

static const db::InstElement &path_nth (const edt::ObjectInstPath *p, unsigned int n) 
{
  //  HINT: this method is not quite fast - the path is a list
  edt::ObjectInstPath::iterator e;
  for (e = p->begin (); e != p->end () && n-- > 0; ++e) 
    ;
  tl_assert (e != p->end ());
  return *e;
}

template <>
gsi::Class<edt::ObjectInstPath> gsi::Class<edt::ObjectInstPath>::decl ("ObjectInstPath", 
  gsi::method ("cv_index", &edt::ObjectInstPath::cv_index, 
    "@brief Accessor to the cellview index that describes which cell view the shape or instance is located in"
  ) + 
  gsi::method ("cell_index", &edt::ObjectInstPath::cell_index_tot, 
    "@brief Accessor to the cell index of the cell that the selection applies to.\n"
    "This method returns the cell index that describes which cell the selected shape is located in or the cell whose instance is selected if \\is_cell_inst? is true."
  ) + 
  gsi::method ("source", &edt::ObjectInstPath::cell_index, 
    "@brief Returns to the cell index of the cell that the selected element resides inside.\n"
    "If this reference represents a cell instance, this method delivers the index of the cell in which the "
    "cell instance resides. Otherwise, this method returns the same value than \\cell_index.\n"
    "This method has been added in version 0.16."
  ) + 
  gsi::method ("trans", &edt::ObjectInstPath::trans_tot, 
    "@brief Accessor to the transformation applicable for the shape.\n"
    "If this object represents a shape, this transformation describes how the selected shape is transformed into the current cell of the cell view.\n"
    "Basically, this transformation is the accumulated transformation over the instantiation path. If the ObjectInstPath represents a cell instance, this includes the transformation of the selected instance as well."
  ) + 
  gsi::method ("source_trans", &edt::ObjectInstPath::trans, 
    "@brief Accessor to the transformation applicable for an instance and shape.\n"
    "If this object represents a shape, this transformation describes how the selected shape is transformed into the current cell of the cell view.\n"
    "If this object represents an instance, this transformation describes how the selected instance is transformed into the current cell of the cell view.\n"
    "This method is similar to \\trans, except that the resulting transformation does not include the instance "
    "transformation if the object represents an instance.\n"
    "This method has been added in version 0.16."
  ) + 
  gsi::method ("layer", &edt::ObjectInstPath::layer, 
    "@brief Accessor to the layer index that describes which layer the selected shape is on\n"
    "This method delivers valid results only for object selections that represent shapes, i.e for "
    "which \\is_cell_inst? is false."
  ) + 
  gsi::method ("shape", &edt::ObjectInstPath::shape, 
    "@brief Accessor to the shape object that describes the selected shape geometrically\n"
    "This method delivers valid results only for object selections that represent shapes, i.e for "
    "which \\is_cell_inst? is false."
  ) + 
  gsi::method_ext ("inst", &edt::inst, 
    "@brief Deliver the instance represented by this selection\n"
    "This method delivers valid results only if \\is_cell_inst? is true.\n"
    "It returns the instance reference (an \\Instance object) that this selection represents.\n"
    "This method has been added in version 0.16."
  ) +
  gsi::method ("is_cell_inst?", &edt::ObjectInstPath::is_cell_inst, 
    "@brief True, if this selection represents a cell instance\n"
    "If this attribute is true, the shape reference and layer are not valid.\n"
  ) + 
  gsi::method ("seq", &edt::ObjectInstPath::seq, 
    "@brief The sequence number\n"
    "The sequence number describes when the item was selected.\n"
    "A sequence number of 0 indicates that the item was selected in the first selection action (without 'Shift' pressed).\n"
  ) + 
  gsi::method_ext ("path_length", &edt::path_length, 
    "@brief Returns the length of the path (number of elements delivered by \\each_inst)\n"
    "This method has been added in version 0.16."
  ) + 
  gsi::method_ext ("path_nth", &edt::path_nth, 
    "@brief Returns the nth element of the path (similar to \\each_inst but with direct access through the index)\n"
    "@args n\n"
    "@param n The index of the element to retrieve (0..\\path_length-1)\n"
    "This method has been added in version 0.16."
  ) + 
  gsi::iterator ("each_inst", (edt::ObjectInstPath::iterator (edt::ObjectInstPath::*) () const) &edt::ObjectInstPath::begin, 
                              (edt::ObjectInstPath::iterator (edt::ObjectInstPath::*) () const) &edt::ObjectInstPath::end, 
    "@brief Yield the instantiation path\n"
    "The instantiation path describes by an sequence of \\InstElement objects the path by which the cell containing the selected shape is found from the cell view's current cell.\n"
    "If this object represents an instance, the path will contain the selected instance as the last element.\n"
    "The elements are delivered top down."
  ),
  "@brief A class describing a selected shape or instance\n"
  "\n"
  "A shape or instance is addressed by a path which describes all instances leading to the specified\n"
  "object. These instances are described through \\InstElement objects, which describe the instance and, in case of array instances, the specific array member.\n"
  "For shapes, additionally the layer and the shape itself is specified. The ObjectInstPath objects\n"
  "encapsulates both forms, which can be distinguished with the \\is_cell_inst? attribute.\n"
);

class SelectionIterator 
{
public:
  typedef edt::Service::objects::value_type value_type;
  typedef edt::Service::objects::const_iterator iterator_type;
  typedef void pointer; 
  typedef const value_type &reference;
  typedef std::forward_iterator_tag iterator_category;
  typedef void difference_type;

  SelectionIterator (const std::vector<edt::Service *> &services) 
    : m_services (services), m_service (0)
  {
    if (! m_services.empty ()) {
      m_iter = m_services [m_service]->selection ().begin ();
      next ();
    }
  }

  bool at_end () const
  {
    return (m_service >= m_services.size ());
  }

  SelectionIterator &operator++ ()
  {
    ++m_iter;
    next ();
    return *this;
  }

  const value_type &operator* () const
  {
    return *m_iter;
  }

private:
  std::vector<edt::Service *> m_services;
  unsigned int m_service;
  iterator_type m_iter;

  void next ()
  {
    while (m_iter == m_services [m_service]->selection ().end ()) {
      ++m_service;
      if (m_service < m_services.size ()) {
        m_iter = m_services [m_service]->selection ().begin ();
      } else {
        break;
      }
    }
  }
};

//  extend the layout view by "edtService" specific methods 

static bool has_object_selection (const lay::LayoutView *view)
{
  std::vector<edt::Service *> edt_services = view->get_plugins <edt::Service> ();
  for (std::vector<edt::Service *>::const_iterator s = edt_services.begin (); s != edt_services.end (); ++s) {
    if ((*s)->has_selection ()) {
      return true;
    }
  }
  return false;
}

static SelectionIterator begin_objects_selected (const lay::LayoutView *view)
{
  return SelectionIterator (view->get_plugins <edt::Service> ());
}


static
gsi::Class<lay::LayoutView> layout_view_decl ("LayoutView", 
  gsi::method_ext ("has_object_selection?", &has_object_selection, 
    "@brief Returns true, if geometrical objects (shapes or cell instances) are selected in this view"
  ) +
  gsi::iterator_ext ("each_object_selected", &begin_objects_selected,
    "@brief Iterate over each selected geometrical object, yielding a \\ObjectInstPath object for each of them"
  ),
  ""
);

}

==> klayout-0.19.3.tar.gz: klayout-0.19.3/src/gsiDeclEdt.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "gsi.h"
#include "edtService.h"
#include "layLayoutView.h"

namespace edt
{

static const db::Instance &inst (const edt::ObjectInstPath *p) 
{
  if (! p->is_cell_inst ()) {
    throw tl::Exception ("Selection does not represent an instance in 'inst' method");
  }
  return p->back ().inst_ptr;
}

static unsigned int path_length (const edt::ObjectInstPath *p)
{
  //  HINT: this method is not quite fast - the path is a list
  return std::distance (p->begin (), p->end ());
}

static const db::InstElement &path_nth (const edt::ObjectInstPath *p, unsigned int n) 
{
  //  HINT: this method is not quite fast - the path is a list
  edt::ObjectInstPath::iterator e;
  for (e = p->begin (); e != p->end () && n-- > 0; ++e) 
    ;
  tl_assert (e != p->end ());
  return *e;
}

template <>
gsi::Class<edt::ObjectInstPath> gsi::Class<edt::ObjectInstPath>::decl ("ObjectInstPath", 
  gsi::method ("cv_index", &edt::ObjectInstPath::cv_index, 
    "@brief Accessor to the cellview index that describes which cell view the shape or instance is located in"
  ) + 
  gsi::method ("cell_index", &edt::ObjectInstPath::cell_index_tot, 
    "@brief Accessor to the cell index of the cell that the selection applies to.\n"
    "This method returns the cell index that describes which cell the selected shape is located in or the cell whose instance is selected if \\is_cell_inst? is true."
  ) + 
  gsi::method ("source", &edt::ObjectInstPath::cell_index, 
    "@brief Returns to the cell index of the cell that the selected element resides inside.\n"
    "If this reference represents a cell instance, this method delivers the index of the cell in which the "
    "cell instance resides. Otherwise, this method returns the same value than \\cell_index.\n"
    "This method has been added in version 0.16."
  ) + 
  gsi::method ("trans", &edt::ObjectInstPath::trans_tot, 
    "@brief Accessor to the transformation applicable for the shape.\n"
    "If this object represents a shape, this transformation describes how the selected shape is transformed into the current cell of the cell view.\n"
    "Basically, this transformation is the accumulated transformation over the instantiation path. If the ObjectInstPath represents a cell instance, this includes the transformation of the selected instance as well."
  ) + 
  gsi::method ("source_trans", &edt::ObjectInstPath::trans, 
    "@brief Accessor to the transformation applicable for an instance and shape.\n"
    "If this object represents a shape, this transformation describes how the selected shape is transformed into the current cell of the cell view.\n"
    "If this object represents an instance, this transformation describes how the selected instance is transformed into the current cell of the cell view.\n"
    "This method is similar to \\trans, except that the resulting transformation does not include the instance "
    "transformation if the object represents an instance.\n"
    "This method has been added in version 0.16."
  ) + 
  gsi::method ("layer", &edt::ObjectInstPath::layer, 
    "@brief Accessor to the layer index that describes which layer the selected shape is on\n"
    "This method delivers valid results only for object selections that represent shapes, i.e for "
    "which \\is_cell_inst? is false."
  ) + 
  gsi::method ("shape", &edt::ObjectInstPath::shape, 
    "@brief Accessor to the shape object that describes the selected shape geometrically\n"
    "This method delivers valid results only for object selections that represent shapes, i.e for "
    "which \\is_cell_inst? is false."
  ) + 
  gsi::method_ext ("inst", &edt::inst, 
    "@brief Deliver the instance represented by this selection\n"
    "This method delivers valid results only if \\is_cell_inst? is true.\n"
    "It returns the instance reference (an \\Instance object) that this selection represents.\n"
    "This method has been added in version 0.16."
  ) +
  gsi::method ("is_cell_inst?", &edt::ObjectInstPath::is_cell_inst, 
    "@brief True, if this selection represents a cell instance\n"
    "If this attribute is true, the shape reference and layer are not valid.\n"
  ) + 
  gsi::method ("seq", &edt::ObjectInstPath::seq, 
    "@brief The sequence number\n"
    "The sequence number describes when the item was selected.\n"
    "A sequence number of 0 indicates that the item was selected in the first selection action (without 'Shift' pressed).\n"
  ) + 
  gsi::method_ext ("path_length", &edt::path_length, 
    "@brief Returns the length of the path (number of elements delivered by \\each_inst)\n"
    "This method has been added in version 0.16."
  ) + 
  gsi::method_ext ("path_nth", &edt::path_nth, 
    "@brief Returns the nth element of the path (similar to \\each_inst but with direct access through the index)\n"
    "@args n\n"
    "@param n The index of the element to retrieve (0..\\path_length-1)\n"
    "This method has been added in version 0.16."
  ) + 
  gsi::iterator ("each_inst", (edt::ObjectInstPath::iterator (edt::ObjectInstPath::*) () const) &edt::ObjectInstPath::begin, 
                              (edt::ObjectInstPath::iterator (edt::ObjectInstPath::*) () const) &edt::ObjectInstPath::end, 
    "@brief Yield the instantiation path\n"
    "The instantiation path describes by an sequence of \\InstElement objects the path by which the cell containing the selected shape is found from the cell view's current cell.\n"
    "If this object represents an instance, the path will contain the selected instance as the last element.\n"
    "The elements are delivered top down."
  ),
  "@brief A class describing a selected shape or instance\n"
  "\n"
  "A shape or instance is addressed by a path which describes all instances leading to the specified\n"
  "object. These instances are described through \\InstElement objects, which describe the instance and, in case of array instances, the specific array member.\n"
  "For shapes, additionally the layer and the shape itself is specified. The ObjectInstPath objects\n"
  "encapsulates both forms, which can be distinguished with the \\is_cell_inst? attribute.\n"
);

class SelectionIterator 
{
public:
  typedef edt::Service::objects::value_type value_type;
  typedef edt::Service::objects::const_iterator iterator_type;
  typedef void pointer; 
  typedef const value_type &reference;
  typedef std::forward_iterator_tag iterator_category;
  typedef void difference_type;

  SelectionIterator (const std::vector<edt::Service *> &services, bool transient) 
    : m_services (services), m_service (0), m_transient_selection (transient)
  {
    if (! m_services.empty ()) {
      m_iter = m_transient_selection ? m_services [m_service]->transient_selection ().begin () : m_services [m_service]->selection ().begin ();
      next ();
    }
  }

  bool at_end () const
  {
    return (m_service >= m_services.size ());
  }

  SelectionIterator &operator++ ()
  {
    ++m_iter;
    next ();
    return *this;
  }

  const value_type &operator* () const
  {
    return *m_iter;
  }

private:
  std::vector<edt::Service *> m_services;
  unsigned int m_service;
  bool m_transient_selection;
  iterator_type m_iter;

  void next ()
  {
    while (m_iter == (m_transient_selection ? m_services [m_service]->transient_selection ().end () : m_services [m_service]->selection ().end ())) {
      ++m_service;
      if (m_service < m_services.size ()) {
        m_iter = m_transient_selection ? m_services [m_service]->transient_selection ().begin () : m_services [m_service]->selection ().begin ();
      } else {
        break;
      }
    }
  }
};

//  extend the layout view by "edtService" specific methods 

static bool has_object_selection (const lay::LayoutView *view)
{
  std::vector<edt::Service *> edt_services = view->get_plugins <edt::Service> ();
  for (std::vector<edt::Service *>::const_iterator s = edt_services.begin (); s != edt_services.end (); ++s) {
    if ((*s)->has_selection ()) {
      return true;
    }
  }
  return false;
}

static bool has_transient_object_selection (const lay::LayoutView *view)
{
  std::vector<edt::Service *> edt_services = view->get_plugins <edt::Service> ();
  for (std::vector<edt::Service *>::const_iterator s = edt_services.begin (); s != edt_services.end (); ++s) {
    if ((*s)->has_transient_selection ()) {
      return true;
    }
  }
  return false;
}

static SelectionIterator begin_objects_selected (const lay::LayoutView *view)
{
  return SelectionIterator (view->get_plugins <edt::Service> (), false);
}

static SelectionIterator begin_objects_selected_transient (const lay::LayoutView *view)
{
  return SelectionIterator (view->get_plugins <edt::Service> (), true);
}


static
gsi::Class<lay::LayoutView> layout_view_decl ("LayoutView", 
  gsi::method_ext ("has_object_selection?", &has_object_selection, 
    "@brief Returns true, if geometrical objects (shapes or cell instances) are selected in this view"
  ) +
  gsi::iterator_ext ("each_object_selected", &begin_objects_selected,
    "@brief Iterate over each selected geometrical object, yielding a \\ObjectInstPath object for each of them"
  ) +
  gsi::method_ext ("has_transient_object_selection?", &has_transient_object_selection,  
    "@brief Returns true, if geometrical objects (shapes or cell instances) are selected in this view in the transient selection\n"
    "\n"
    "The transient selection represents the objects selected when the mouse hovers over the "
    "layout windows. This selection is not used for operations but rather to indicate which object would "
    "be selected if the mouse is clicked.\n"
    "\n"
    "This method was introduced in version 0.18."
  ) +
  gsi::iterator_ext ("each_object_selected_transient", &begin_objects_selected_transient,
    "@brief Iterate over each geometrical objects in the transient selection, yielding a \\ObjectInstPath object for each of them\n"
    "\n"
    "This method was introduced in version 0.18."
  ),
  ""
);

}
